<?php

/* by Pieter Hordijk */
function generate_password($length) {
  $chars =  'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz'.
            '0123456789``-=~!@#$%^&*()_+,./<>?;:[]{}\|';

  $str = '';
  $max = strlen($chars) - 1;

  for ($i=0; $i < $length; $i++)
    $str .= $chars[rand(0, $max)];

  return $str;
}

/**
Validate an email address.
Provide email address (raw input)
Returns true if the email address has the email
address format and the domain exists.

By Douglas Lovell
*/
function validEmail($email)
{
   $isValid = true;
   $atIndex = strrpos($email, "@");
   if (is_bool($atIndex) && !$atIndex)
   {
      $isValid = false;
   }
   else
   {
      $domain = substr($email, $atIndex+1);
      $local = substr($email, 0, $atIndex);
      $localLen = strlen($local);
      $domainLen = strlen($domain);
      if ($localLen < 1 || $localLen > 64)
      {
         // local part length exceeded
         $isValid = false;
      }
      else if ($domainLen < 1 || $domainLen > 255)
      {
         // domain part length exceeded
         $isValid = false;
      }
      else if ($local[0] == '.' || $local[$localLen-1] == '.')
      {
         // local part starts or ends with '.'
         $isValid = false;
      }
      else if (preg_match('/\\.\\./', $local))
      {
         // local part has two consecutive dots
         $isValid = false;
      }
      else if (!preg_match('/^[A-Za-z0-9\\-\\.]+$/', $domain))
      {
         // character not valid in domain part
         $isValid = false;
      }
      else if (preg_match('/\\.\\./', $domain))
      {
         // domain part has two consecutive dots
         $isValid = false;
      }
      else if (!preg_match('/^(\\\\.|[A-Za-z0-9!#%&`_=\\/$\'*+?^{}|~.-])+$/',
                 str_replace("\\\\","",$local)))
      {
         // character not valid in local part unless
         // local part is quoted
         if (!preg_match('/^"(\\\\"|[^"])+"$/',
             str_replace("\\\\","",$local)))
         {
            $isValid = false;
         }
      }
      if ($isValid && !(checkdnsrr($domain,"MX") || checkdnsrr($domain,"A")))
      {
         // domain not found in DNS
         $isValid = false;
      }
   }
   return $isValid;
}

function replace_slashes($partname)
{
    $name = $partname;
    while (($idx = strpos($name, '/'))) {
        if ($idx == 0)
			$name = "-" . substr($name, $idx + 1);
		else
			$name = substr($name, 0, $idx) . "-" . substr($name, $idx + 1);
	}
	while (($idx = strpos($name, ' '))) {
		if ($idx == 0)
			$name = "_" . substr($name, $idx + 1);
		else
			$name = substr($name, 0, $idx) . "_" . substr($name, $idx + 1);
	}
    return $name;
}

function findpart($db, $datatable, $usertable, $partname, $userid, $groupid)
{
    $result = NULL;
    if ($groupid == 0) {
        $query = $db->prepare("SELECT symbol_id FROM $datatable WHERE partname=:name AND user_id=:user");
        if ($query->execute(array(":name" => $partname, ":user" => $userid)))
            $result = $query->fetch();
    } else {
        $query = $db->prepare("SELECT symbol_id FROM $datatable JOIN $usertable ON $datatable.user_id=$usertable.user_id WHERE partname=:name AND group_id=:group");
        if ($query->execute(array(":name" => $partname, ":group" => $groupid)))
            $result = $query->fetch();
    }
    $query->closeCursor();
    return $result;
}

?>