<?php

error_reporting (E_ALL | E_STRICT);
include "configuration.php";
include "functions.php";

if (count($_POST) == 0) {
    include "showinfo.php";
    exit();
}

if (!isset($_POST['cmd']))
    exit("ERROR: no command");

$usertable = TABLE_PREFIX . "users";

/* special commands are handled first, because they do not have a category */
if ($_POST['cmd'] == "user") {
    if (!isset($_POST['user']) || strlen($_POST['user']) == 0)
        exit("ERROR: username is required");
    $query = $db->prepare("SELECT user_id FROM $usertable WHERE username=:username");
    if ($query->execute(array(':username' => $_POST['user']))) {
        if (($row = $query->fetch()))
            $userid = $row[0];
    }
    $query->closeCursor();
    if (!isset($_POST['email']) || strlen($_POST['email']) == 0) {
        /* just check whether the user exists */
        if (isset($userid))
            print("ok\n");
        else
            print("unknown\n");
    } else {
        /* add a user */
        if (isset($userid))
            exit("ERROR: user exists");
        if (!validEmail($_POST['email']))
            exit("ERROR: invalid email address");
        /* generate a password */
        $pwd = generate_password(8);
        /* store the new user */
        $query = $db->prepare("INSERT INTO $usertable (username, userpwd, email, group_id, timestamp) VALUES (:name, :pwd, :email, 0, :tstamp)");
        if (!$query->execute(array(":name" => $_POST['user'], ":pwd" => md5($pwd), ":email" => $_POST['email'], ":tstamp" => time())))
            exit("ERROR: failure adding the user");

        /* send it by email */
        $subject = "KiCad repository account";
        $body = "Dear ".$_POST['user'].",\r\n\r\n"
              . "Your registration for the KiCad repository has completed."
              . " Your password is \"$pwd\" (without the double quotes).\r\n\r\n"
              . "To be able to access the KiCad repository, please enter"
              . " the password in the \"Remote Repository\" dialog in the"
              . " KiCad Librarian.\r\n\r\n"
              . "Regards,\r\n"
              . "The KiCad Librarian team\r\n";
        $headers = "From: KiCad Repository <".EMAIL_ADDRESS.">\r\n"
                 . "Reply-To: ".EMAIL_ADDRESS."\r\n"
                 . "X-Mailer: PHP/".phpversion()."\r\n"
                 . "Message-ID: <".time().$_POST['user'].rand()."@".$_SERVER['SERVER_NAME'].">\r\n";
        //??? for non-public server, send the email to a moderator (instead of to the user)
        if (!mail($_POST['email'], $subject, $body, $headers))
            exit("ERROR: failure sending the mail");
        print("ok\n");
    }
    exit();
}

/* all other commands need a category */
if (!isset($_POST['cat']) || ($_POST['cat'] != "symbols" && $_POST['cat'] != "footprints" && $_POST['cat'] != "models"))
    exit("ERROR: no category or invalid category");
$datatable = TABLE_PREFIX . $_POST['cat'];

/* with the exception of commands "list" and "get" (and command "user", which
   was already handled), all commands require a username;
   in a non-public repository, commands "list" and "get" also require a user
   name  */
if (!PUBLIC_REPOS || ($_POST['cmd'] != "list" && $_POST['cmd'] != "get")) {
    if (!isset($_POST['user']) || strlen($_POST['user']) == 0)
        exit("ERROR: username is required");
    $query = $db->prepare("SELECT user_id, group_id FROM $usertable WHERE username=:username");
    if ($query->execute(array(':username' => $_POST['user']))) {
        if (($row = $query->fetch())) {
            $userid = $row[0];
            $groupid = $row[1];
        }
    }
    if (!isset($userid))
        exit("ERROR: authetication failed.");
    if (!isset($groupid) || is_null($groupid))
        $groupid = 0;
}

/* some commands require a username + password, when non-public all commands require this */
if (!PUBLIC_REPOS || $_POST['cmd'] == "put" || $_POST['cmd'] == "putinfo" || $_POST['cmd'] == "del") {
    if (!isset($_POST['pwd']) || strlen($_POST['pwd']) == 0)
        exit("ERROR: username and password are required");
    $query = $db->prepare("SELECT userpwd FROM $usertable WHERE user_id=:userid");
    if ($query->execute(array(':userid' => $userid)) && ($row = $query->fetch())) {
        $pwd = md5($_POST['pwd']);
        if ($pwd != $row[0])
            exit("ERROR: authetication failed.");
    } else {
        exit("ERROR: authetication failed.");
    }
    $query->closeCursor();
}

if ($_POST['cmd'] == "list") {
    $filter = "";
    if (isset($_POST['filter']) && strlen($_POST['filter']) > 0) {
        $filter = "WHERE partname LIKE \"%" . $_POST['filter'] . "%\"";
        $filter .= " OR $datatable.description LIKE \"%" . $_POST['filter'] . "%\"";
        $filter .= " OR $datatable.keywords LIKE \"%" . $_POST['filter'] . "%\"";
        if ($_POST['cat'] == "symbols")
            $filter .= " OR $datatable.aliases LIKE \"%" . $_POST['filter'] . "%\"";
    }
    $query = $db->prepare("SELECT partname,username FROM $datatable JOIN $usertable ON $datatable.user_id=$usertable.user_id $filter");
    if ($query->execute()) {
        while (($row = $query->fetch())) {
            print('"' . $row[0] . '","' . $row[1] . '"' . "\r\n");
        }
        $query->closeCursor();
    }
} else {
    if (!isset($_POST['part']) || strlen($_POST['part']) == 0)
        exit("ERROR: no part name");

    if ($_POST['cmd'] == "get") {
        /* get author name (or use the name of the current user, if no author is provided) */
        if (!isset($_POST['author']) || strlen($_POST['author']) == 0) {
            if (!isset($_POST['user']) || strlen($_POST['user']) == 0)
                exit("ERROR: author name is required");
            $authorname = $_POST['user'];
        } else {
            $authorname = $_POST['author'];
        }
        $query = $db->prepare("SELECT data FROM $datatable JOIN $usertable ON $datatable.user_id=$usertable.user_id WHERE partname=:name AND username=:author");
        if ($query->execute(array(":name" => $_POST['part'], ":author" => $authorname))) {
            $row = $query->fetch();
            $query->closeCursor();
            if ($row)
                print(stripslashes($row[0]));
            else
                exit("ERROR: part not found");
        }
    } else if ($_POST['cmd'] == "put") {
        /* the author name is ignored, we always use the user name as the author */
        if (!isset($_POST['data']) || strlen($_POST['data']) == 0)
            exit("ERROR: no part data");
        $data = addslashes($_POST['data']);
        $row = findpart($db, $datatable, $usertable, $_POST['part'], $userid, $groupid);
        if ($row) {
            /* part exists, update */
            $query = $db->prepare("UPDATE $datatable SET user_id=:user, data=:data, timestamp=:tstamp WHERE symbol_id:symid");
            if (!$query->execute(array(":user" => $userid, ":data" => $data, ":symid" => $row[0], ":tstamp" => time())))
                exit("ERROR: failure modifying the part");
        } else {
            /* new part */
            $query = $db->prepare("INSERT INTO $datatable (user_id, partname, data, timestamp) VALUES (:user, :name, :data, :tstamp)");
            if (!$query->execute(array(":user" => $userid, ":name" => $_POST['part'], ":data" => $data, ":tstamp" => time())))
                exit("ERROR: failure adding the part");
        }
        print("ok\n");
    } else if ($_POST['cmd'] == "del") {
        $row = findpart($db, $datatable, $usertable, $_POST['part'], $userid, $groupid);
        if ($row) {
            $query = $db->prepare("DELETE FROM $datatable WHERE symbol_id=:symid");
            $query->execute(array(":symid" => $row[0]));
        } else {
            exit("ERROR: failure deleting part (part not found)");
        }
        /* the VRML model may be used by multiple parts, and it is therefore not
           deleted; this creates "orphaned" models over time */
        print("ok\n");
    } else if ($_POST['cmd'] == "putinfo") {
        $row = findpart($db, $datatable, $usertable, $_POST['part'], $userid, $groupid);
        if ($row) {
            if (isset($_POST['descr']) && strlen($_POST['descr']) > 0) {
                $data = addslashes($_POST['descr']);
                $query = $db->prepare("UPDATE $datatable SET description=:data, timestamp=:tstamp WHERE symbol_id=:symid");
                $query->execute(array(":data" => $data, ":tstamp" => time(), ":symid" => $row[0]));
            }
            if (isset($_POST['alias']) && strlen($_POST['alias']) > 0) {
                $data = addslashes($_POST['alias']);
                $query = $db->prepare("UPDATE $datatable SET aliases=:data, timestamp=:tstamp WHERE symbol_id=:symid");
                $query->execute(array(":data" => $data, ":symid" => $row[0], ":tstamp" => time()));
            }
            if (isset($_POST['fplist']) && strlen($_POST['fplist']) > 0) {
                $data = addslashes($_POST['fplist']);
                $query = $db->prepare("UPDATE $datatable SET fplist=:data, timestamp=:tstamp WHERE symbol_id=:symid");
                $query->execute(array(":data" => $data, ":symid" => $row[0], ":tstamp" => time()));
            }
            if (isset($_POST['keywords']) && strlen($_POST['keywords']) > 0) {
                $data = addslashes($_POST['keywords']);
                $query = $db->prepare("UPDATE $datatable SET keywords=:data, timestamp=:tstamp WHERE symbol_id=:symid");
                $query->execute(array(":data" => $data, ":tstamp" => time(), ":symid" => $row[0]));
            }
            if (isset($_POST['pitch']) && strlen($_POST['pitch']) > 0) {
                $data = (float)($_POST['pitch']);
                $query = $db->prepare("UPDATE $datatable SET pitch=:data, timestamp=:tstamp WHERE symbol_id=:symid");
                $query->execute(array(":data" => $data, ":symid" => $row[0], ":tstamp" => time()));
            }
            if (isset($_POST['span']) && strlen($_POST['span']) > 0) {
                $data = (float)($_POST['span']);
                $query = $db->prepare("UPDATE $datatable SET span=:data, timestamp=:tstamp WHERE symbol_id=:symid");
                $query->execute(array(":data" => $data, ":symid" => $row[0], ":tstamp" => time()));
            }
            if (isset($_POST['pins']) && strlen($_POST['pins']) > 0) {
                $data = (int)($_POST['pins']);
                $query = $db->prepare("UPDATE $datatable SET pins=:data, timestamp=:tstamp WHERE symbol_id=:symid");
                $query->execute(array(":data" => $data, ":symid" => $row[0], ":tstamp" => time()));
            }
            if (isset($_POST['thumb']) && strlen($_POST['thumb']) > 0) {
                $data = explode(",", $_POST['thumb']);  /* split description from data */
                /* get the extension from the description */
                $ext = "png";   /* preset (default) */
                $start = strpos($data[0], "/");
                $end = strpos($data[0], ";");
                if ($start >= 0 && $end > $start)
                    $ext = substr($data[0], $start + 1, $end - $start - 1);
                /* make the path and filename where to store the image */
                $root = $_SERVER['SCRIPT_FILENAME'];
                $idx = strrpos($root, "/");
                if ($idx >= 0)
                    $root = substr($root, 0, $idx);
                $cat = substr($_POST['cat'], 0, strlen($_POST['cat']) - 1);
                $basename = $_POST['part'];
                $basename = str_replace("/", "-", $basename);
                $basename = str_replace("\\", "-", $basename);
                $basename = str_replace(" ", "_", $basename);
                $path = $root . "/bitmaps/" . $cat . "_" . $basename . "." . $ext;
                /* when Base64 gets posted, all '+' are converted to ' ' by PHP
                   (it assumes URL encoding), so we need to change them back */
                $pixels = str_replace(" ", "+", $data[1]);
                $pixels = base64_decode($pixels);
                $fp = fopen($path, "wb");
                fwrite($fp, $pixels);
                fclose($fp);
            }
        } else {
            exit("ERROR: failure updating part (part not found)");
        }
        print("ok\n");
    }
}

?>
